from django.conf import settings
from imp import find_module
from importlib import import_module
import django_rq
import rq_scheduler


def build_cron_string(minutes=None, hours=None, days=None, months=None, weekday=None):
    if hours is not None:
        if minutes is None:
            minutes = 0

    if days is not None:
        if hours is None:
            hours = 0

            if minutes is None:
                minutes = 0

    if months is not None:
        if days is None:
            days = 1

            if hours is None:
                hours = 0

                if minutes is None:
                    minutes = 0

    return ' '.join(
        (
            minutes is None and '*' or str(minutes),
            hours is None and '*' or str(hours),
            days is None and '*' or str(days),
            months is None and '*' or str(months),
            weekday is None and '*' or str(weekday)
        )
    )


def get_scheduler(queue_name=None):
    return rq_scheduler.Scheduler(
        queue_name=queue_name or 'default',
        connection=django_rq.get_connection()
    )


def autodiscover():
    for app in settings.INSTALLED_APPS:
        import_module(app)
        name = '%s.cron' % app

        try:
            import_module(name)
        except ImportError as ex:
            try:
                find_module(name)
            except ImportError:
                continue

            raise ex  # pragma: no cover



from .decorators import daily, hourly, interval, weekly
from .helpers import autodiscover


__version__ = '2.1.0'
__all__ = (
    'autodiscover',
    'daily',
    'hourly',
    'interval',
    'weekly'
)



from logging import getLogger
from .helpers import build_cron_string, get_scheduler


def _register(queue_name, cron_string):
    scheduler = get_scheduler(queue_name)
    logger = getLogger('podiant.cron')

    def wrapper(f):
        scheduler.cron(
            cron_string,
            func=f,
            queue_name=queue_name
        )

        logger.debug(
            'Scheduled job \'%s\' with cron string \'%s\'' % (
                f.__name__,
                cron_string
            )
        )

        return f

    return wrapper


def interval(
    minutes=None,
    hours=None,
    days=None,
    months=None,
    weekday=None,
    queue_name=None,
    timeout='30s'
):
    cron_string = build_cron_string(minutes, hours, days, months, weekday)
    return _register(queue_name, cron_string)


def weekly(weekday=0, queue_name=None, timeout='30s'):
    cron_string = build_cron_string(weekday=0)
    return _register(queue_name, cron_string)


def daily(queue_name=None, timeout='30s'):
    cron_string = build_cron_string(hours=0)
    return _register(queue_name, cron_string)


def hourly(queue_name=None, timeout='30s'):
    cron_string = build_cron_string(minutes=0)
    return _register(queue_name, cron_string)



from django.core.management.base import BaseCommand
from logging import getLogger
from ...helpers import autodiscover, get_scheduler


class Command(BaseCommand):
    help = 'Run scheduled jobs'

    def handle(self, *args, **options):
        logger = getLogger('podiant.cron')

        scheduler = get_scheduler()
        jobs = list(scheduler.get_jobs())

        if any(jobs):
            logger.debug(
                'Clearing %d job(s) from scheduler' % len(jobs)
            )

            for job in jobs:
                logger.debug('Deleting \'%s\'' % job.func_name)
                job.delete()

        autodiscover()

        logger.debug('Running cron worker')
        scheduler.run()
